#version 150

#moj_import <fog.glsl>

uniform float GameTime;
uniform vec4 ColorModulator;
uniform float FogStart;
uniform float FogEnd;
uniform vec4 FogColor;
uniform vec2 ScreenSize;

in vec3 position;
in mat4 ProjInv;
in float isSky;
in float vertexDistance;

out vec4 fragColor;

#define GRIDOFFSET 0.078
#define GRIDDENSITY 10.0

void main() {

    float twinkleSpeed = 0.5;
    float starTwinkle = sin((GameTime * 2000 * twinkleSpeed) + position.x + position.z) * 255;
    vec4 finalColor = ColorModulator;

    float morningness = length(finalColor.rgb - vec3(121, 166, 255) / 255.0);
    morningness = smoothstep(0.08, 0.4, morningness);
    float eveningness = length(finalColor.rgb - vec3(79, 108, 166) / 255.0);
    eveningness = smoothstep(0.08, 0.6, eveningness);
    float nightness = length(finalColor.rgb - vec3(0, 0, 0));
    nightness = smoothstep(0.02, 0.08, nightness);
      
    vec3 dayColor = vec3(81, 176, 255) / 255;
    vec3 nightColor = vec3(5, 9, 26) / 255;
    vec3 eveningColor = vec3(85, 89, 106) / 255;
    vec3 thunderColor = vec3(35, 39, 56) / 255;

    vec4 skyColor = vec4(mix(dayColor, mix(eveningColor, mix(nightColor, thunderColor, nightness), eveningness), morningness), finalColor.a);
    
    if (finalColor.a < 1) { 
        skyColor = (vec4(127, 184, 255, starTwinkle) / 255);
    }

    // get player space view vector
    vec4 screenPos = gl_FragCoord;
    screenPos.xy = (screenPos.xy / ScreenSize - vec2(0.5)) * 2.0;
    screenPos.zw = vec2(1.0);
    vec3 view = normalize((ProjInv * screenPos).xyz);

    // custom fog calculation if sky because sky disc is no longer above the head
    if (isSky > 0.5) {
        float ndusq = clamp(dot(view, vec3(0.0, 1.0, 0.0)), 0.0, 1.0);
        ndusq = ndusq * ndusq;

        fragColor = linear_fog(skyColor, pow(1.0 - ndusq, 8.0), 0.0, 1.0, FogColor);
    } 
    // default shading for void plane and stars
    else {
        fragColor = linear_fog(skyColor, vertexDistance, FogStart, FogEnd, FogColor);
    }

}